require 'rails_helper'

module IntegrationPal
  describe Api::V1::JobsController, :type => :controller do
    routes { IntegrationPal::Engine.routes }

    context 'unauthorized' do
      describe '#index' do
        let(:job) { create(:integration_pal_job) }

        it 'restricts access to authorized users' do
          get :show, params: {id: job.to_param}
          expect(response.status).to eq(401)
        end
      end
    end

    context 'authorized' do
      let!(:worker) { create(:integration_pal_worker) }

      describe '#show' do
        let(:job) { create(:integration_pal_job) }
        let(:controller_path) { "/api/v1/jobs/#{job.id}"}

        before(:each) do
          allow_any_instance_of(ActionDispatch::TestRequest).to receive(:fullpath).and_return(controller_path)
          request.env["HTTP_ACCEPT"] = "application/json"
          ApiAuth.sign!(request, worker.access_id, worker.secret_key)
        end

        it 'returns workers as @workers' do
          get :show, params: {id: job.to_param}
          expect(response.body).to eq job.to_json
        end
      end

      describe "POST #create" do
        before(:each) do
          controller.class.skip_before_action :authenticate_api rescue nil
        end

        context "with valid params" do
          let(:valid_attributes) {
            build(:integration_pal_job).attributes
          }

          it "creates a new Job" do
            expect {
              post :create, params: {job: valid_attributes}
            }.to change(Job, :count).by(1)
          end

          it "assigns a newly created job as @job" do
            post :create, params: {job: valid_attributes}
            expect(assigns(:job)).to be_a(Job)
            expect(assigns(:job)).to be_persisted
          end
        end

        context "with invalid params" do
          let(:invalid_attributes) {
            build(:integration_pal_job, worker_id: nil).attributes
          }

          it "doesn't create the job" do
            expect {
              post :create, params: {job: invalid_attributes}
            }.to change(Job, :count).by(0)
          end
        end
      end
    end
  end
end